Package: fields
Version: 17.3
Date: 2026-05-03
Title: Tools for Spatial Data
Authors@R: c( person("Douglas", "Nychka", role = c("aut", "cre"),
               email = "douglasnychka@gmail.com"),
            person("Reinhard", "Furrer", role = c("aut"),
               email = "reinhard.furrer@math.uzh.ch"),
            person("John", "Paige", role = c("aut"),
               email = "paigejo@uw.edu"),
            person("Stephan", "Sain", role = "aut",
               email = "sainsr2@gmail.com"),
	     person("Florian", "Gerber", role = "aut",
               email = "flora.fauna.gerber@gmail.com"),
	     person("Matthew", "Iverson", role = "aut",
               email = "miverson@mines.edu"),
	     person("Rider", "Johnson", role = "aut",
               email = "riderjohnson@mines.edu")
	       ) 
Maintainer: Douglas Nychka <douglasnychka@gmail.com>
Description: For curve, surface and function fitting with an emphasis
 on splines, spatial data, geostatistics,  and spatial statistics. The major methods
 include cubic, and thin plate splines, Kriging, and compactly supported
 covariance functions for large data sets. The splines and Kriging methods are
 supported by functions that can determine the smoothing parameter
 (nugget and sill variance) and other covariance function parameters by cross
 validation and also by restricted maximum likelihood. For Kriging
 there is an easy to use function that also estimates the correlation
 scale (range parameter).  A major feature is that any covariance function
 implemented in R and following a simple format can be used for
 spatial prediction. There are also many useful functions for plotting
 and working with spatial data as images. This package also contains
 an implementation of sparse matrix methods for large spatial data
 sets and currently requires the sparse matrix (spam) package. Use
 help(fields) to get started and for an overview. All graphics functions
 focus on using  base R graphics.  The fields source
 code is deliberately commented and provides useful explanations of
 numerical details as a companion to the manual pages. The commented
 source code can be viewed by expanding the source code version
 and looking in the R subdirectory. The reference for fields can be generated
 by the citation function in R and has DOI <doi:10.5065/D6W957CT>. Development
 of this package was supported in part by the National Science Foundation  Grant
 1417857,  the National Center for Atmospheric Research, and Colorado School of Mines.
 See the Fields URL
 for a vignette on using this package and some background on spatial statistics.
License: GPL (>= 2)
URL: https://github.com/dnychka/fieldsRPackage
Depends: R (>= 4.0.0), methods, spam, viridisLite, RColorBrewer
Imports: maps
Suggests: mapproj
NeedsCompilation: yes
Repository: CRAN
Packaged: 2026-05-04 12:41:59 UTC; nychka@mines.edu
Author: Douglas Nychka [aut, cre],
  Reinhard Furrer [aut],
  John Paige [aut],
  Stephan Sain [aut],
  Florian Gerber [aut],
  Matthew Iverson [aut],
  Rider Johnson [aut]
Date/Publication: 2026-05-05 12:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-05 16:39:27 UTC; unix
Archs: fields.so.dSYM
