exportPattern("^[^\\.]")

importFrom("grDevices", "chull", "col2rgb", "colors", "palette", "rgb",
             "trans3d")
importFrom("graphics", "abline", "arrows", "axis", "box", "boxplot",
             "contour", "hist", "lines", "locator", "matlines",
             "matplot", "mtext", "pairs", "par", "persp", "plot",
             "points", "polygon", "rect", "segments", "text", "title",
             "grid", "legend")
importFrom("stats", "D", "approx", "coef", "cor", "dist", "fft", "mad",
             "median", "optim", "optimize", "predict", "quantile",
             "rnorm", "sd", "var","qchisq","pnorm")
importFrom("utils", "object.size")
importFrom("maps","map")
importFrom("viridisLite","viridis")
importFrom("stats", "qnorm")
importFrom("stats", "lsfit")


importFrom("RColorBrewer", "brewer.pal")

S3method("[",spatialDesign)
S3method(coef, Krig)
S3method(fitted, Krig)
S3method(image, plot)
S3method(image, smooth)
S3method(print, Krig)
S3method(print, mKrig)
S3method(print,spatialDesign)
S3method(print,spatialProcess)
S3method(print,sreg)
S3method(print,qsreg)
S3method(print,summary.Krig)
S3method(print,summarySpatialDesign)
S3method(print,mKrigSummary)
S3method(print,spatialProcessSummary)
S3method(print,summary.sreg)
S3method(summary,Krig)
S3method(summary,sreg)
S3method(summary,ncdf)
S3method(summary,qsreg)
S3method(summary,spatialDesign)
S3method(summary,spatialProcess)
S3method(summary,mKrig)
S3method(plot,Krig)
S3method(plot,spatialProcess)
S3method(plot,surface)
S3method(plot,spatialDesign)
S3method(plot,sreg)
S3method(plot,qsreg)
S3method(plot,vgram.matrix)
S3method(plot,vgram)
S3method(surface,Krig)
S3method(surface,mKrig)
S3method(surface,default)
S3method(predict,fastTps)
S3method(predict,interp.surface) 
S3method(predict,Krig)
S3method(predict,mKrig) 
S3method(predict,qsreg)
S3method(predict,sreg)
S3method(predict,surface)
S3method(predict,surface.default) 
S3method(predict,Tps)
S3method(predictSE,Krig)
S3method(predictSE,mKrig)
S3method(predictSurface,default)
S3method(predictSurface,fastTps)
S3method(predictSurface,Krig)
S3method(predictSurface,mKrig)
S3method(predictSurfaceSE,default)
useDynLib(fields,.registration=TRUE)

import("spam", "methods")


