# fields  is a package for analysis of spatial data written for
# the R software environment .
# Copyright (C) 2017
# University Corporation for Atmospheric Research (UCAR)
# Contact: Douglas Nychka, nychka@ucar.edu,
# National Center for Atmospheric Research, PO Box 3000, Boulder, CO 80307-3000
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the R software environment if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# or see http://www.r-project.org/Licenses/GPL-2    
fastTps.MLE <- function(x, y, weights = rep(1, nrow(x)), 
                        Z = NULL, ..., par.grid=NULL, theta, lambda = NULL,
                        lambda.profile = TRUE, 
    verbose = FALSE, relative.tolerance = 1e-04) {
  
  warning("fastTps.MLE is deprecated and might be removed in a future release.  Use fastTpsMLE instead.")
  
  do.call("fastTpsMLE", c(list(x, y, weights, Z), list(...), list(par.grid, theta, lambda, 
                                                                  lambda.profile, verbose, 
                                                                  relative.tolerance)))
}

