%#
%# fields  is a package for analysis of spatial data written for
%# the R software environment.
%# Copyright (C) 2022 Colorado School of Mines
%# 1500 Illinois St., Golden, CO 80401
%# Contact: Douglas Nychka,  douglasnychka@gmail.edu,
%#
%# This program is free software; you can redistribute it and/or modify
%# it under the terms of the GNU General Public License as published by
%# the Free Software Foundation; either version 2 of the License, or
%# (at your option) any later version.
%# This program is distributed in the hope that it will be useful,
%# but WITHOUT ANY WARRANTY; without even the implied warranty of
%# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%# GNU General Public License for more details.
%#
%# You should have received a copy of the GNU General Public License
%# along with the R software environment if not, write to the Free Software
%# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
%# or see http://www.r-project.org/Licenses/GPL-2
%##END HEADER
%##END HEADER

\name{minitri}
\alias{minitri}
\title{
  Mini triathlon results
}
\description{
Results from a mini triathlon sponsored by Bud Lite, held in
Cary, NC, June 1990. Times are in minutes for the male 30-34 
group. Man was it hot and humid! (DN)

The events in  order were
swim:  (1/2 mile)
bike:  (15 miles)
run:   (4 miles)

<s-section 
name= "DATA DESCRIPTION">
This is a dataframe. Row names are the place within this age group based 
on total time.  
}
\arguments{
\item{swim}{
swim times 
}
\item{bike}{
bike times 
}
\item{run}{
run times 
}
}
\keyword{datasets}
% docclass is data
% Converted by Sd2Rd version 1.21.
